package article_test

import (
	"testing"
	"unsafe"
)

type Foo struct { // 16 bytes
	A int8 // align = 1
	// padding = 1
	B int16 // align = 2
	// padding = 4 (not 6. remember that the
	// next field just needs to align with 8 byte boundary,
	// and we already have 2 bytes from field `A`)
	C int64 // align = 8
}

type Bar struct { // 16 byte
	A int64 // align = 8
	B int16 // align = 2
	C int8  // align = 1
	// padding = 5
	// we need to add 5 bytes so that Bar is a multiple of
	// the alignment of a struct
}

type Baz struct { // 24 bytes
	A int16 // align = 2
	// padding = 6
	B int64 // align = 8
	C int8  // align = 1
	// padding = 7
}

type Quux struct { // 4 bytes
	A int16 // align = 2
	B int8  // align = 1
          // padding = 1
}

func TestStructs(t *testing.T) {
	var t1 Foo
	var t2 Bar
	var t3 Baz
	var t4 Quux

	// 以下のコードでt1...t4をinterface{}に格納すると
	// 値が変わってくるのでご注意ください

	t.Logf("t1: Size = %d, Alignment = %d", unsafe.Sizeof(t1), unsafe.Alignof(t1))
	t.Logf("t2: Size = %d, Alignment = %d", unsafe.Sizeof(t2), unsafe.Alignof(t2))
	t.Logf("t3: Size = %d, Alignment = %d", unsafe.Sizeof(t3), unsafe.Alignof(t3))
	t.Logf("t4: Size = %d, Alignment = %d", unsafe.Sizeof(t4), unsafe.Alignof(t4))
}
